#! /usr/local/bin/perl
# %%%{CotEditorXInput=Selection}%%%
# %%%{CotEditorXOutput=ReplaceSelection}%%%

use Encode;

my $INPUT=`cat -`;

# 桁揃えに使用する文字列
my $THE_MARK = "#";

# タブ幅単位で桁揃えする
my $USE_TABWIDTH = 1;

# 最後のマークを対象とする。同じ行に、マークが何度も出現する場合に、最後を対象とする場合は1を、最初のマークを対象とする場合は0を指定する。ゲストブックにて yosito さんが指摘していただいたものを反映しました。yosito さん、ありがとうございました。
my $USE_LASTMARK = 0;

# **============================================================
# * 記号で揃える.
# * $THE_MARK で指定された記号を左揃えする。
# * 例：
# *  before:
# *          $int_foo = 123; # the integer Foo
# *          $string_bar = "234";  # the string Bar
# *
# *  after:
# *          $int_foo = 123;       # the integer Foo
# *          $string_bar = "234";  # the string Bar
# *
# * @author      odoru-saboten
# * @author      Tachi.K
# * @since       2011/03/02
# * @version     2012/02/16
# * @param       CotEditorXInput=Selection
# * @return      CotEditorXOutput=ReplaceSelection
# ============================================================

# ============================================================
# THE_MARK をユーザに確認する
# ============================================================
my $cmd = 'osascript ';
$cmd .= ' -e "on run THEMARK"';
$cmd .= ' -e "tell application \"System Events\""';
$cmd .= ' -e "activate"';
$cmd .= ' -e "try"';
$cmd .= ' -e "set ret to display dialog \"Please specify the mark.\" default answer THEMARK"';
$cmd .= ' -e "set buttonReturned to button returned of ret"';
$cmd .= ' -e "if buttonReturned = \"OK\" then"';
$cmd .= ' -e "set returnmark to text returned of ret"';
$cmd .= ' -e "end if"';
$cmd .= ' -e "on error"';
$cmd .= ' -e "set returnmark to \"\""';
$cmd .= ' -e "end try"';
$cmd .= ' -e "end tell"';
$cmd .= ' -e "tell application \"CotEditor\""';
$cmd .= ' -e "activate"';
$cmd .= ' -e "end tell"';
$cmd .= ' -e "return returnmark"';
$cmd .= ' -e "end run"';
$cmd .= " \"$THE_MARK\"";

$THE_MARK = readpipe $cmd;
chomp $THE_MARK;

if ($THE_MARK eq "") {
  print $INPUT;
  exit(0);
}

# ============================================================
# THE_MARK 以前の最大幅を得る
# ============================================================
$maxColumn = 0;
foreach my $line ( split(/\n/, $INPUT)) { 
  $firstPart = "";
  
  # THE_MARK 前後を得る
  if ($USE_LASTMARK) {
    # 最後の THE_MARK を使用する
    if ($line =~ /(.+)($THE_MARK\b[^$THE_MARK\b]+)$/os) {
      $firstPart = $1;
    }
  } else {
    # 最初の THE_MARK を使用する
    if ($line =~ /$THE_MARK/os) {
      $firstPart = $`;
    }
  }
  print "$firstPart\n";
  if ($firstPart ne "") {
    
    # 前半の長さを得る
    $firstPartEUC = $firstPart;
    Encode::from_to($firstPartEUC, "utf8", "euc-jp" );
    $lengthOfFirstPart = length($firstPartEUC);
    
    # 最大値を得る
    if ($maxColumn < $lengthOfFirstPart) {
      $maxColumn = $lengthOfFirstPart;
    }
  }
}

if ($USE_TABWIDTH) {
  # タブ幅を取得する
  $THE_TABWIDTH = readpipe 'defaults read com.aynimac.CotEditor tabWidth';
  if ($maxColumn % $THE_TABWIDTH != 0) {
    # タブ幅単位で桁揃えする
    $maxColumn += $THE_TABWIDTH - ($maxColumn % $THE_TABWIDTH);
  }
}

# ============================================================
# 最大幅に従って空白を設定する
# ============================================================
foreach my $line ( split(/\n/, $INPUT)) { 
  $firstPart = "";

  # THE_MARK 前後を得る
  if ($USE_LASTMARK) {
    # 最後の THE_MARK を使用する
    if ($line =~ /(.+)($THE_MARK[^$THE_MARK]+)$/os) {
      $firstPart  = $1;
      $secondPart = $2;
    }
  } else {
    # 最初の THE_MARK を使用する
    if ($line =~ /$THE_MARK/os) {
      $firstPart  = $`;
      $secondPart = $&.$'; #'
    }
  }

  if ($firstPart ne "") {
    
    # 前半の長さを得る
    $firstPartEUC = $firstPart;
    Encode::from_to($firstPartEUC, "utf8", "euc-jp" );
    $lengthOfFirstPart = length($firstPartEUC);
    
    # 必要なスペースを得る
    $lengthOfSpace = $maxColumn - $lengthOfFirstPart;
    $spc = "";
    for ($i = 0; $i < $lengthOfSpace; $i++) {
      $spc .= " ";
    }
    
    print "$firstPart$spc$secondPart\n";
  } else {
    print "$line\n";
  }
}











